package de.fitko.fitconnect.client;

import de.fitko.fitconnect.api.domain.auth.OAuthToken;
import de.fitko.fitconnect.api.services.Subscriber;

import java.util.Optional;

/**
 * A fluent client for handing in a subscription for a {@link Subscriber}
 */
public class SubscriberClient extends Client {

    private SubscriberClient() {}

     public static Authenticate builder() {
         final Subscriber subscriber = getService(Subscriber.class);
         return new ClientBuilder(subscriber);
    }

    public interface Authenticate {
        /**
         *
         * @param clientId
         * @param secret
         * @param scope
         * @return
         */
        AttachmentUpload authenticate(String clientId, String secret, String... scope);
    }

    public interface AttachmentUpload {
        void uploadAttachments();
    }

    public static class ClientBuilder implements Authenticate, AttachmentUpload{

        private Subscriber subscriber;
        private Optional<OAuthToken> token;

        private ClientBuilder(Subscriber subscriber) {
            this.subscriber = subscriber;
        }

        @Override
        public AttachmentUpload authenticate(String clientId, String secret, String... scope) {
            this.token = subscriber.retrieveOAuthToken(clientId, secret, scope);
            return this;
        }

        @Override
        public void uploadAttachments() {
            checkIfAuthenticated();
            //metadataService.createMetadata();
        }

        private void checkIfAuthenticated(){
            if(this.token == null || this.token.isEmpty()){
                throw new IllegalStateException("Not authenticated, please authenticate first");
            }
        }
    }
}